
/*

USART routines for Microchip PIC17C4x
Designed by Shane Tolmie
Compiled with Hi-Tech C v7.83pl3

*/

#include	<pic.h>
#include	"codes.h"
#include	"always.h"
#include	"serial.h"
#include	"equates.h"

void init_serial(void)
{
	//***********
	//setup USART

	//TXSTA register
	TX9D = 0;				//9th-bit of transmit data
	TRMT = 0;				//transmit shift register empty bit
	SYNC = 0;				//0 for asynchronous mode
	TXEN = 1;				//transmit enable
	TX9  = 0;				//9-bit transmit enable - 0 for 8-bit transmission
	CSRC = 0;				//don't care in asynchronous mode

	RCREG = RCREG;	//clear flags from receive.
	SPBRG = 25;			//9600 baud at 16 MHz

	//RCSTA register
	RX9D = 0;				//9-bit of receive data
	OERR = 0;				//overrun error
	FERR = 0;				//framing error - updated by readin RCREG
	CREN = 1;				//1 for continuous receive
	SREN = 0;				//single receive enable
	RX9  = 0;				//0 for 8-bit reception
	SPEN = 1;				//1 enable serial port pins

	clear_usart_errors();
}

void putch(char c)
{
	while(!TXIF)					//set when register is empty
	{
	}
	TXREG=c;
}

void putst(register const char * str)
{
	while((*str)!=0)
	{
		putch(*str);
		str++;
	}
}

//gets a character from the serial port without timeout
unsigned char getch(void)
{

	OERR=0;					//must software clear overrun error or else all reception dies if 2-byte buffer overruns
	while(!RCIF)
	{
	}
	return RCREG;
}

void clear_usart_errors(void)
{
	static unsigned char dummy;

	if (OERR==1) 
	{
		CREN=0;												//0->1 clears framing error
		CREN=1;		
	}
	if (FERR==1)
	{
		dummy=RCREG;									//clears overrun error
	}
}

/*
writes a character to the serial port in hex
if serial lines are disconnected, there are no errors
*/

void putchhex(unsigned char c)
{
	unsigned char temp;
	
	// transmits in hex

	temp=c;

	c=(c >> 4);
	if (c<10) c+=48; else c+=55;
	putch(c);

	c=temp;

	c=(c & 0x0F);
	if (c<10) c+=48; else c+=55;
	putch(c);
}

void putinthex(unsigned int c)
{
	#define ramuint(x)						(*((unsigned int *) (x)))
	#define ramuint_hibyte(x)			(*(((unsigned char *)&x)+1))
	#define ramuint_lobyte(x)			(*(((unsigned char *)&x)+0))
	#define ramuchar(x)						(*((unsigned char *) (x)))

	putchhex(ramuint_hibyte(c));	
	putchhex(ramuint_lobyte(c));

	#undef ramuint(x)
	#undef ramuint_hibyte(x)
	#undef ramuint_lobyte(x)
	#undef ramuchar(x)
}

//if there has been a previous timeout error from getch_timeout, this returns TRUE
unsigned char usart_timeout(void)
{
//	return usart_timeout_error;
	return FALSE;
}



/*
writes a character to the serial port in decimal
if serial lines are disconnected, there are no errors
*/
void putchdec(unsigned char c)
{
	unsigned char temp;
	
	temp=c;
	//hundreds
	if ((c/100)>0) putch((c/100)+'0');
	c-=(c/100)*100;

	//tens
	if (((temp/10)>0) || ((temp/100)>0)) putch((c/10)+'0');
	c-=(c/10)*10;

	//ones
	putch((c/1)+'0');
}
